# Event Log Track Encrypt

## Presentation

Event Log Track Encrypt add support for encrypted logs
to [Events Log Track](https://www.drupal.org/project/events_log_track) module (
ELT)

This feature is useful for protecting, without loss, certain sensitive data that
may be stored in ELT logs (e.g. the modification of a configuration entity
containing credentials).

The module
uses [public key cryptography](https://en.wikipedia.org/wiki/Public-key_cryptography)
to encrypt data: The data is encrypted with a public key before being recorded
in the ELT log. Only the owner of the private key corresponding to the public
key is able to decrypt the data.

Only the 'description' part of an ELT log is encrypted.

The module provides [drush](https://www.drush.org) commands for decrypting data,
but no graphical user interface.

## Prerequisites

Event Log Track Encrypt
requires [PHP OpenSSL support](https://www.php.net/manual/en/openssl.installation.php)
to be enabled on your PHP version.

You will need
the [OpenSSL utility](https://github.com/openssl/openssl?tab=readme-ov-file#welcome-to-the-openssl-project)
to generate your public/private key pair.

Event Log Track Encrypt requires the following modules:

- [Events Log Track](https://www.drupal.org/project/events_log_track)
- [Encrypt](https://www.drupal.org/project/encrypt)
- [Key](https://www.drupal.org/project/key)

## Installation

### Key generation

First, you need to generate the public/private key pair.

On a machine
running [the OpenSSL utility](https://github.com/openssl/openssl?tab=readme-ov-file#welcome-to-the-openssl-project)

```bash
openssl genpkey \
-algorithm RSA \
-pkeyopt rsa_keygen_bits:2048 \
-aes256 \
-out event_log_track_private_key.pem \
&& \
openssl rsa \
-in event_log_track_private_key.pem \
-pubout \
-out event_log_track_public_key.pem
```

This command will :

- Generate a private key in the file `event_log_track_private_key.pem`.
- Ask you to choose a password for your private key.
- Generate the public key corresponding to the private key in
  the `event_log_track_public_key.pem` file.

The private key generated by this command has a size of 2048 bits, uses the
algo [RSA](https://en.wikipedia.org/wiki/RSA_(cryptosystem)) and is encrypted
with [AES](https://en.wikipedia.org/wiki/Advanced_Encryption_Standard).
We advise you not to modify these generation criteria, as Event Log Track
Encrypt module is configured by default to use this type of key.

**Warning:**

- Keep your private key (the `event_log_track_private_key.pem` file) and
  password in a safe place.
- Anyone who knows the private key and its password will be able to decrypt data
  encrypted with the public key.
- If you lose the private key or its password, you will no longer be able to
  decrypt data encrypted with the public key.

### Install and configure the module

- Activate Event Log Track Encrypt module.
- Accept the installation
  of [Events Log Track](https://www.drupal.org/project/events_log_track) / [Encrypt](https://www.drupal.org/project/encrypt) / [Key](https://www.drupal.org/project/key)
  modules if offered.
- Configure the public key:
    - Go to the key administration interface (/admin/config/system/keys):
      Administration > Configuration > System > Keys
    - Edit the key named `Event log track public key` by clicking on the '
      Modify' button.
    - Confirm the warning message (Be extremely careful when editing a key!...)
      by clicking on the `Edit` button.
    - In the `Key value` field, copy **your public key**. Warning: copy your *
      *public key**, not your private key!
    - Save the modification by clicking on the `Save` button.
- Enable encryption:
    - Go to the ELT module administration interface (
      /admin/config/system/events-log-track): Administration > Configuration >
      System > Event log track
    - In the `Encryption` section:
        - Check `Enable encryption` box
        - Select the type of events to be encrypted (e.g. `config`).
    - Save changes by clicking on the `Save configuration` button.

## How it works

When you enable encryption, only the `description` part of the ELT log is
encrypted.

This results in logs like :

```
ELT [config] [my_module.settings] [save] ON [admin/config/my-module]
BY [user:1:admin:authenticated,administrator] [@@crypt@@Uw4yPREqqwTpWST/LPiYipdU
kgMkXBqQHQP5+JZZEexreffRJkIwjgbU=,G2/jd99GIIHsMZiqMmngE3tfcx08VHRfZAHnVmPAYBMG7K
g==,ehqi+gIJyp1dpDAsReCoAg==@@endcrypt@@]
```

In the example above, the ELT log tells us that the configuration
entity `my_module.settings` has been saved by the user `admin` from the
url `admin/config/my-module`.

Only the `description` part, which contains the modifications made to the
entity, is encrypted.

The encrypted part of the log is encapsulated between the `@@crypt@@`
and `@@endcrypt@@` tags.
This tagging makes it easier to locate the encrypted strings.
The drush decryption commands are based on this markup.

Technically :

- The encryption method is based
  on [openssl_seal()](https://www.php.net/manual/en/function.openssl-seal.php),
  which allows data to be sealed using a public key.
- The implementation is strongly inspired by
  the [encrypt_rsa](https://www.drupal.org/project/encrypt_rsa) module and
  its [PublicOpenSslSealEncryptionMethod](https://git.drupalcode.org/project/encrypt_rsa/-/blob/8.x-1.x/src/Plugin/EncryptionMethod/PublicOpenSslSealEncryptionMethod.php?ref_type=heads)
  class.

## Log decryption

The event_log_track_encrypt module provides drush commands for decrypting ELT
logs.

```bash
root@c218ae432fd3:/var/www/html/web/sites/default# drush list
[...]
elt:                                                                                                                                        
  elt:decrypt-db (eltddb)         Drush command to decrypt database event logs                                       
  elt:decrypt-file (eltdf)        Drush command to decrypt log file                                                  
  elt:decrypt-string (eltds)      Drush command to decrypt a string                                                                                   
[...]
```

- The `elt:decrypt-db` command queries the `event_log_track` table, searches the
  query result for encrypted strings and decrypts them with the private key.
- The `elt:decrypt-file` command searches the supplied log file for encrypted
  strings and decrypts them using the private key.
- The command `elt:decrypt-string` attempts to decrypt the supplied string with
  the private key. The string must not contain the tags `@@crypt@@`
  and `@@endcrypt@@`.

These 3 commands take as parameters the path to your **private key
** (`--private-key-file`) and will ask you to enter the password for your
private key.

For security reasons, when entering the password, no characters are displayed in
the console.

To see details of the options available by command, use `drush help [command]`.

Examples:

Decrypt a string :

```
# drush eltds --private-key-file=/var/keys/event_log_track_private_key.pem Uw4yP
REqqwTpWST/LPiYipdUkgMkXBqQHQP5+JZZEexreffRJkIwjgbU=,G2/jd99GIIHsMZiqMmngE3tfcx0
8VHRfZAHnVmPAYBMG7Kg==,ehqi+gIJyp1dpDAsReCoAg==

 Passphrase (to unlock private key)::
 > 

my clear data
```

Decrypt the log file /tmp/example.log

```
# drush eltdf --private-key-file=/var/keys/event_log_track_private_key.pem /tmp/
example.log 

 Passphrase (to unlock private key)::
 > 

[...]
172.23.0.1 - - [07/May/2024:15:02:06 +0000] "GET /admin/config/development/log_s
tdout HTTP/1.1" 200 8067 "https://test.com/admin/config" "Firefox"
ELT [config] [event_log_track.settings] [save] ON [admin/config/development/log_
stdout] BY [user:1:admin:authenticated,administrator] [new:#012  stdout:#012    
format: 'ELT [[event-log:type]] [[event-log:ref_char]] [[event-log:operation]] O
N [[event-log:path]] BY [user:[event-log:user:uid]:[event-log:user:name]:[event-
log:user:roles:join:,]] [[event-log:description]]'#012    output_type: stdout#01
2old:#012  stdout:#012    format: 'ELT [[event-log:type]] [[event-log:ref_char]]
 [[event-log:operation]] ON [[event-log:path]] BY [user:[event-log:user:uid]:[ev
 ent-log:user:name]:[event-log:user:roles:join:,]] [[event-log:description]]'#01
 2    output_type: watchdog#012]
172.23.0.1 - - [07/May/2024:15:02:10 +0000] "POST /admin/config/development/log_
stdout HTTP/1.1" 303 1004 "https://test.com/admin/config/development/log_stdout"
 "Firefox"
172.23.0.1 - - [07/May/2024:15:02:11 +0000] "GET /admin/config/development/log_s
tdout HTTP/1.1" 200 8250 "https://test.com/admin/config/development/log_stdout" 
"Firefox"
[...]
```

Decrypt the entire `event_log_track` table

```
# drush eltddb --private-key-file=/var/keys/event_log_track_private_key.pem

 Passphrase (to unlock private key)::
 > 

1 | cache_clear | cache_clear | admin/config/development/performance | 1 | admin
 | Cache cleared | 1 | 172.23.0.1 | 1715094108
2 | config | save | admin/config/development/log_stdout |  | event_log_track.set
tings | new:#012  stdout:#012    format: 'ELT [[event-log:type]] [[event-log:ref
_char]] [[event-log:operation]] ON [[event-log:path]] BY [user:[event-log:user:u
id]:[event-log:user:name]:[event-log:user:roles:join:,]] [[event-log:description
]]'#012    output_type: stdout#012old:#012  stdout:#012    format: 'ELT [[event-
log:type]] [[event-log:ref_char]] [[event-log:operation]] ON [[event-log:path]] 
BY [user:[event-log:user:uid]:[event-log:user:name]:[event-log:user:roles:join:,
]] [[event-log:description]]'#012    output_type: watchdog#012 | 1 | 172.23.0.1 
| 1715094130
[...]
```

Decrypt selected elements in the `event_log_track` table: The first 3 “config
save” events made from IP 172.23.0.1

```
# drush eltddb --private-key-file=/var/keys/event_log_track_private_key.pem --co
ndition="type LIKE config" --condition="operation LIKE save" --condition="ip LIK
E 172.23.0.1" --limit=3

 Passphrase (to unlock private key)::
 > 

86 | config | save | admin/config/system/events-log-track |  | event_log_track.e
ncrypt.settings | new:#012  event_types_to_encrypt:#012    authentication: false
#012    authorization: false#012    cache_clear: false#012    comment: false#012
    config: true#012    file: false#012    group: true#012    media: false#012  
  menu: false#012    node: false#012    taxonomy: false#012    user: false#012  
  workflows: false#012old:#012  event_types_to_encrypt:#012    authentication: f
alse#012    authorization: false#012    cache_clear: false#012    comment: false
#012    config: false#012    file: false#012    group: true#012    media: false#
012    menu: false#012    node: false#012    taxonomy: false#012    user: false#
012    workflows: false#012 | 1 | 172.23.0.1 | 1715089619
89 | config | save | admin/config/system/events-log-track |  | event_log_track.e
ncrypt.settings | new:#012  event_types_to_encrypt:#012    authentication: false
#012    authorization: false#012    cache_clear: true#012    comment: false#012 
   config: true#012    file: false#012    group: false#012    media: false#012  
  menu: false#012    node: false#012    taxonomy: false#012    user: false#012  
  workflows: false#012old:#012  event_types_to_encrypt:#012    authentication: f
alse#012    authorization: false#012    cache_clear: false#012    comment: false
#012    config: true#012    file: false#012    group: true#012    media: false#0
12    menu: false#012    node: false#012    taxonomy: false#012    user: false#0
12    workflows: false#012 | 1 | 172.23.0.1 | 1715094050
91 | config | save | admin/config/development/log_stdout |  | event_log_track.se
ttings | new:#012  stdout:#012    format: 'ELT [[event-log:type]] [[event-log:re
f_char]] [[event-log:operation]] ON [[event-log:path]] BY [user:[event-log:user:
uid]:[event-log:user:name]:[event-log:user:roles:join:,]] [[event-log:descriptio
n]]'#012    output_type: stdout#012old:#012  stdout:#012    format: 'ELT [[event
-log:type]] [[event-log:ref_char]] [[event-log:operation]] ON [[event-log:path]]
 BY [user:[event-log:user:uid]:[event-log:user:name]:[event-log:user:roles:join:
 ,]] [[event-log:description]]'#012    output_type: watchdog#012 | 1 | 172.23.0.
 1 | 1715094130
```
